// Generated by CoffeeScript 2.3.1
module.exports = function(BasePlugin) {
  var HighlightjsPlugin, balUtil, cache, extendr, hljs;
  // Requires
  balUtil = require('bal-util');
  extendr = require('extendr');
  hljs = require('highlight.js');
  cache = {};
  return HighlightjsPlugin = (function() {
    var extractLanguage;

    // Define Plugin
    class HighlightjsPlugin extends BasePlugin {
      // Highlight an element
      highlightSource(opts) {
        var aliases, className, config, docpad, err, escape, i, language, len, next, ref, removeIndentation, replaceTab, result, source, transform, transforms;
        // Prepare
        docpad = this.docpad;
        ({source, language, next} = opts);
        config = extendr.extend({}, this.getConfig(), opts.config || {});
        ({escape, replaceTab, aliases, transforms, removeIndentation, className} = config);
        if (removeIndentation !== false) {
          // Remove Indentation
          source = balUtil.removeIndentation(source);
        }
        // Extrat language
        language = extractLanguage(language);
        // Highlight
        result = source;
        if (language !== 'no-highlight') {
          // Correctly escape the source
          if (escape !== true) {
            // Unescape the output as highlightjs always escape
            source = require('he').decode(source);
          }
          ref = transforms || [];
          // Transforms
          for (i = 0, len = ref.length; i < len; i++) {
            transform = ref[i];
            if (transform instanceof Function) {
              // transforms = (source) ->
              source = transform(source, language);
            } else if (transform instanceof Array && transform.length === 2) {
              // transforms = ['find' or RegExp, 'replace']
              source = source.replace(transform[0], transform[1]);
            }
          }
          if (replaceTab) {
            // Replace tabs
            hljs.fixMarkup(source, replaceTab);
          }
          try {
            // Correct aliases
            // Highlight
            if (language && aliases[language]) {
              language = aliases[language];
            }
            // Perform the render
            if (language) {
              if (hljs.getLanguage(language)) {
                result = hljs.highlight(language, source);
              } else {
                docpad.warn(`Highlight.js: Syntax highlight definition does not exist for language: ${language}\nYour probably want to add an alias for this language to one that we do support\nMore information here: https://github.com/docpad/docpad-plugin-highlightjs#language-aliases`);
                result = hljs.highlightAuto(source);
              }
            } else {
              result = hljs.highlightAuto(source);
            }
            // Extract the result
            language = result.language;
            result = result.value;
          } catch (error) {
            err = error;
            if (err) {
              return next(err);
            }
          }
        }
        // Handle
        result = `<pre class="${className}"><code class="hljs ${language}">${result}</code></pre>`.replace(/\t/g, replaceTab);
        next(null, result);
        return this;
      }

      // Render the document
      renderDocument(opts, next) {
        var cacheKey, cacheResult, className, extension, file, plugin, ref;
        ({extension, file} = opts);
        ({className} = this.getConfig());
        plugin = this;
        // Handle
        if (file.type === 'document' && extension === 'html') {
          // Check cache
          cacheKey = opts.content + ((ref = file.attributes.plugins) != null ? ref.highlightjs : void 0);
          cacheResult = cache[cacheKey];
          if (cacheResult) {
            opts.content = cacheResult;
            return next();
          }
          // Grab all the code blocks
          return balUtil.replaceElementAsync({
            html: opts.content,
            element: 'pre',
            removeIndentation: false,
            replace: function(outerHTML, elementNameMatched, attributes, innerHTML, replaceElementCompleteCallback) {
              var classes;
              // Check
              classes = balUtil.getAttribute(attributes, 'class') || '';
              if (classes.indexOf(className) !== -1) {
                return replaceElementCompleteCallback(null, outerHTML);
              }
              // Replace
              return balUtil.replaceElementAsync({
                html: innerHTML,
                element: 'code',
                removeIndentation: false,
                replace: function(outerHTML, elementNameMatched, attributes, innerHTML, replaceElementCompleteCallback) {
                  var ref1;
                  classes = balUtil.getAttribute(attributes, 'class') || '';
                  return plugin.highlightSource({
                    source: innerHTML,
                    language: classes,
                    config: (ref1 = file.attributes.plugins) != null ? ref1.highlightjs : void 0,
                    next: replaceElementCompleteCallback
                  });
                },
                next: replaceElementCompleteCallback
              });
            },
            next: function(err, result) {
              if (err) {
                return next(err);
              }
              opts.content = result;
              cache[cacheKey] = result;
              return next();
            }
          });
        } else {
          return next();
        }
      }

    };

    // Plugin name
    HighlightjsPlugin.prototype.name = 'highlightjs';

    // Plugin configuration
    HighlightjsPlugin.prototype.config = {
      replaceTab: '    ',
      transforms: null,
      escape: false,
      removeIndentation: false,
      className: 'highlight',
      aliases: {
        coffee: 'coffeescript',
        rb: 'ruby',
        js: 'javascript',
        html: 'xml',
        htm: 'xml',
        eco: 'xml',
        erb: 'xml',
        phtml: 'xml'
      }
    };

    // Extract language
    extractLanguage = function(str) {
      var matches;
      // Prepare
      str = (str || '').toLowerCase();
      if (/no[-]?highlight|none/i.test(str)) {
        // No highlighting
        return 'no-highlight';
      }
      // Return the last listed language class
      matches = str.match(/lang(?:uage)?-\w+/g);
      if (matches) {
        return matches.pop().match(/lang(?:uage)?-(\w+)/)[1];
      }
      if (str.indexOf(' ') === -1) {
        // Return the string if it is it
        return str;
      }
      // Auto-highlighting
      return '';
    };

    return HighlightjsPlugin;

  }).call(this);
};
